/* Copyright (C) |Meso|Star> 2016-2017 (contact@meso-star.com)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>. */

#include "ssf.h"
#include "test_ssf_utils.h"

#include <rsys/double3.h>
#include <star/ssp.h>

int
main(int argc, char** argv)
{
  struct mem_allocator allocator;
  struct ssf_bsdf* bsdf;
  struct ssf_bsdf* dummy;
  struct ssf_fresnel* F;
  struct ssf_microfacet_distribution* D;
  struct ssp_rng* rng;
  double N[4]; /* xyz, pdf */
  double wo[4]; /* xyz, pdf */
  size_t i, NSTEPS=100000;
  (void)argc, (void)argv;

  mem_init_proxy_allocator(&allocator, &mem_default_allocator);
  CHECK(ssp_rng_create(&allocator, &ssp_rng_threefry, &rng), RES_OK);
  CHECK(ssf_bsdf_create(&allocator, &ssf_microfacet2_reflection, &bsdf), RES_OK);
  CHECK(ssf_bsdf_create(&allocator, &bsdf_dummy, &dummy), RES_OK);
  CHECK(ssf_fresnel_create(&allocator, &ssf_fresnel_no_op, &F), RES_OK);
  CHECK(ssf_microfacet_distribution_create
    (&allocator, &ssf_beckmann_distribution, &D), RES_OK);

  CHECK(ssf_beckmann_distribution_setup(D, 0.9), RES_OK);

  CHECK(ssf_microfacet_reflection_setup(NULL, NULL, NULL), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(bsdf, NULL, NULL), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(NULL, F, NULL), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(bsdf, F, NULL), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(NULL, NULL, D), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(bsdf, NULL, D), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(NULL, F, D), RES_BAD_ARG);
  CHECK(ssf_microfacet_reflection_setup(bsdf, F, D), RES_OK);

  /* Check energy conservation */
  ssp_ran_sphere_uniform(rng, N);
  ssp_ran_hemisphere_cos(rng, N, wo);
  FOR_EACH(i, 0, NSTEPS) {
    double wi[3], pdf;
    int type;
    CHECK(ssf_bsdf_sample(bsdf, rng, wo, N, wi, &type, &pdf), 1);
    CHECK(type, SSF_GLOSSY | SSF_REFLECTION);
    CHECK(d3_dot(wi, N) > 0, 1);
  }

  CHECK(ssf_bsdf_ref_put(bsdf), RES_OK);
  CHECK(ssf_bsdf_ref_put(dummy), RES_OK);
  CHECK(ssf_fresnel_ref_put(F), RES_OK);
  CHECK(ssf_microfacet_distribution_ref_put(D), RES_OK);
  CHECK(ssp_rng_ref_put(rng), RES_OK);

  check_memory_allocator(&allocator);
  mem_shutdown_proxy_allocator(&allocator);
  CHECK(mem_allocated_size(), 0);
  return 0;
}

